#!/bin/sh
. /etc/iproute2/script/gateway-mgt-function
. /etc/iproute2/script/policy_routing_common_function

SYNONETD_TOOL="/usr/syno/sbin/synonetdtool"
INTERFACE_UP_DOWN_LOCK="/var/run/lock/interface_up_down_${IFNAME}.lock"
INTERFACE_UP_DOWN_TIME_LOCK="/var/run/lock/interface_up_down_time_record_${IFNAME}.lock"
INTERFACE_UP_DOWN_TIME_RECORD="/var/run/lock/interface_up_down_time_record_${IFNAME}"

add_interface()
{
	local GATEWAY="NULL"
	local DNS="NULL"
	local IFCFG_FILE="/etc/sysconfig/network-scripts/ifcfg-${IFNAME}"

	if [ ! -f ${IFCFG_FILE} ]; then
		return
	fi

	local BOOTPROTO=`get_key_value ${IFCFG_FILE} BOOTPROTO`
	if [ "${BOOTPROTO}" = "none" ]; then
		return
	fi

	is_3glte_interface ${IFNAME}
	# 3g/lte dongle's DHCP behavior is handled by synousbmodemd,
	# then we do not need to deal with 3g/lte in link_up hook
	if [ $? -ne 0 -a "${BOOTPROTO}" = "dhcp" ]; then
		if [ "running" == "`/sbin/status dhcp-client IFACE=${IFNAME} | grep -o running`" ]; then
			restart dhcp-client IFACE=${IFNAME}
		else
			start dhcp-client IFACE=${IFNAME}
		fi
	elif [ "${BOOTPROTO}" = "static" ]; then
		GATEWAY=$(get_key_value "${IFCFG_FILE}" GATEWAY)
		if test -z "$GATEWAY" ; then
			GATEWAY="NULL"
		fi
		DNS=$(get_key_value "${IFCFG_FILE}" DNS)
		if test -z "$DNS" ; then
			DNS="NULL"
		fi
	fi

	${SYNONETD_TOOL} --add-gateway-info -4 -2 ${IFNAME} ${GATEWAY} ${DNS} ethernet
	${SYNONETD_TOOL} --refresh-gateway -4

	if [ "${GATEWAY}" != "NULL" ]; then
		local enable_multi_gateway=`/bin/get_key_value /etc/synoinfo.conf multi_gateway`
		if [ "xyes" = "x${enable_multi_gateway}" ]; then
			local IPADDR=`get_key_value ${IFCFG_FILE} IPADDR`
			${SYNONETD_TOOL} --add-policy-route-rule -4 multi-gateway ${IFNAME} NULL NULL ${GATEWAY}
			${SYNONETD_TOOL} --refresh-route-table -4 ${IFNAME} ${IPADDR}
		fi
	fi
}

add_interface_v6()
{
	local GATEWAY="NULL"
	local DNS="NULL"
	local IFCFG_FILE="/etc/sysconfig/network-scripts/ifcfg-${IFNAME}"
	if [ ! -f ${IFCFG_FILE} ]; then
		return
	fi
	local IPV6INIT=`get_key_value ${IFCFG_FILE} IPV6INIT`
	if [ "${IPV6INIT}" = "off" ]; then
		return
	fi

	if [ "${IPV6INIT}" = "static" ]; then
		GATEWAY=`get_key_value ${IFCFG_FILE} IPV6_DEFAULTGW`
		if test -z "$GATEWAY" ; then
			GATEWAY="NULL"
		fi
		DNS=`get_key_value ${IFCFG_FILE} IPV6DNS`
		if test -z "$DNS" ; then
			DNS="NULL"
		fi
	fi

	if [ "${IPV6INIT}" != "static" ]; then
		/usr/bin/rdisc6 -q -n ${IFNAME} -r 1 -w 0
	fi

	${SYNONETD_TOOL} --add-gateway-info -6 -2 ${IFNAME} ${GATEWAY} ${DNS} ethernet
	${SYNONETD_TOOL} --refresh-gateway -6

	if [ "${GATEWAY}" != "NULL" ]; then
		local enable_multi_gateway=`/bin/get_key_value /etc/synoinfo.conf multi_gateway`
		if [ "xyes" = "x${enable_multi_gateway}" ]; then
			local NUM=`/sbin/ifconfig ${IFNAME} | grep inet6 | wc -l`
			local IPADDR=""
			for i in `seq 1 $NUM`
			do
				local IP=`ifconfig eth1 | grep inet6 | sed -n ${i}p | awk '{print $3}'| cut -d "/" -f1`
				if [ ! -z "$IP" ]; then
					if [ $i -eq 1 ]; then
						IPADDR=${IP}
					else
						IPADDR="${IPADDR},${IP}"
					fi
				fi
			done
			if [ ! -z "$IPADDR" ]; then
				${SYNONETD_TOOL} --add-policy-route-rule -6 multi-gateway ${IFNAME} NULL NULL ${GATEWAY}
				${SYNONETD_TOOL} --refresh-route-table -6 ${IFNAME} ${IPADDR}
			fi
		fi
	fi
}

replace_netif_boot_proto() {
	local usbmodeminfos=`ls -d /tmp/usbmodem.info*`
	local usbmodeminfos_num=`ls -d /tmp/usbmodem.info* | wc -l`
	if [ 0 -eq $usbmodeminfos_num ]; then
		return
	fi

	for info in $usbmodeminfos; do
		local iface=`/bin/get_key_value $info USBNET_IFNAME`
		local network_iface_conf="/etc/sysconfig/network-scripts/ifcfg-$iface"
		/usr/syno/bin/synosetkeyvalue $network_iface_conf BOOTPROTO dhcp
	done
}

start_dslite()
{
	synonetdtool --start-dslite ${IFNAME} &
}

start_mape()
{
	synonetdtool --start-mape ${IFNAME}
}

start_ipv6_router()
{
	if [ -e "/etc/sysconfig/networking/ipv6-${IFNAME}" ]; then
		local IP=`/sbin/ifconfig ${IFNAME} | grep -Eo 'inet addr:([0-9]*\.){3}[0-9]*' | cut -d: -f 2`
		${SYNONETD_TOOL} --init-ipv6-router "${IFNAME}" "${IP}" &
	fi
}

main()
{
	ifconfig ${IFNAME} > /dev/null 2>&1
	if [ $? -ne 0 ]; then
		return
	fi

	is_3glte_interface ${IFNAME}
	if [ $? -eq 0 ]; then
		replace_netif_boot_proto
		add_interface
		/etc/rc.network update-nat &
		return
	fi

	is_topology_wan_interface ${IFNAME}
	local WIRED_INTERFACE=$?
	if [ $WIRED_INTERFACE -eq 0 ]; then
		add_interface
		add_interface_v6
		start_ipv6_router
		start_mape
		start_dslite
	fi
	/etc/rc.network update-nat &
}

case $1 in
	--sdk-mod-ver)
	#Print SDK support version
	echo "1.0";
	;;
	--name)
	#Print package name
	echo "SynorouterClient"
	;;
	--pkg-ver)
	#Print package version
	echo "1.0";
	;;
	--vendor)
	#Print package vendor
	echo "Synology";
	;;
	--post)

	if [ "WAN" != "${IFTYPE}" ]; then
		exit
	fi

	now="$(awk '/^now/ {print $3}' /proc/timer_list)"
	/bin/mkdir "$(/usr/bin/dirname "${INTERFACE_UP_DOWN_LOCK}")"
	output="$( (
	i=0
	while [ 30 != "$i" ]; do
		if flock -n -x 9; then
			if [ "${now}" != "$(cat "${INTERFACE_UP_DOWN_TIME_RECORD}")" ]; then
				echo "${now}" > "${INTERFACE_UP_DOWN_TIME_RECORD}"
				echo "Success"
			fi
			break
		else
			i="$((i+1))"
			sleep 1
		fi
	done
	)9> "${INTERFACE_UP_DOWN_TIME_LOCK}")"
	if [ "${output}" != "Success" ]; then
		exit
	fi
	(
	i=0
	while [ 120 != "$i" ]; do
		if flock -n -x 10; then
			(
			j=0
			while [ 30 != "$j" ]; do
				if flock -n -x 9; then
					if [ "${now}" != "$(cat "${INTERFACE_UP_DOWN_TIME_RECORD}")" ]; then
						logger -p err -t "tool/interfaceUpAdjustList.sh" "Skip ${IFNAME} up"
						break
					fi
					flock -u 9
					main
					break
				else
					j="$((j+1))"
					sleep 1
				fi
			done
			)9> "${INTERFACE_UP_DOWN_TIME_LOCK}"
			break
		else
			i="$((i+1))"
			sleep 1
		fi
	done
	)10> "$INTERFACE_UP_DOWN_LOCK"
	;;
	*)
	echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
	;;
esac

